export type LightUser = {
  name: string;
  id: string;
};

export type Player = {
  user: LightUser;
  rating: number;
  ratingDiff: number;
};

export type Players = {
  white: Player;
  black: Player;
};

export type Clock = {
  initial: number;
  increment: number;
  totalTime: number;
};

export type Game = {
  id: string;
  rated: boolean;
  variant: string;
  speed: string;
  perf: string;
  createdAt: number;
  lastMoveAt: number;
  status: string;
  players: Players;
  winner?: string;
  moves?: string;
  tournament?: string;
  clock: Clock;
};

export class ParsedGame {
  id: string;
  variant: string;
  result: string;
  white: string;
  black: string;
  decoratedWhite: string;
  decoratedBlack: string;
  gameInfo: string;
  status: string;
  moves: string[];
  clock: Clock;

  constructor(game: Game) {
    this.id = game.id;
    this.clock = game.clock;
    this.variant = game.variant;
    this.status = game.status;
    this.result = '1/2-1/2';
    if (game.winner) {
      this.result = game.winner === 'white' ? '1-0' : '0-1';
    }
    this.white = game.players.white.user.name;
    this.black = game.players.black.user.name;
    this.decoratedWhite = `${this.white} ( ${game.players.white.rating} )`;
    this.decoratedBlack = `${this.black} ( ${game.players.black.rating} )`;
    this.gameInfo = `${this.decoratedWhite} - ${this.decoratedBlack} ${this.result} [ ${this.clock.initial} + ${this.clock.increment} ] ${this.variant} [ ${this.status} ]`;
    this.moves = [];
    if (game.moves) {
      this.moves = game.moves.split(' ');
    }
  }
}
